#if defined (PWM_PIN)
// Управление ШИМ на пинах
/*
 * Обрабатываются команды
 * P_ON       - включит ШИМ на максимум (1024)
 * P_OFF      - передает на ШИМ 0
 * PWM 0 xxx  - передает на ШИМ xxx
 * PWM 0 +    - увеличивает ШИМ на 1
 * PWM 0 -    - уменьшает ШИМ на 1
 * Если номер PWM больше или равен количеству действует на все
 */

// Для ESP-01 Пин 0 - GPIO0, 2 - GPIO2  {0} , {0,2}
uint8_t  pin_pwm[PWM_PIN]={0,2};
uint16_t signal_pwm[PWM_PIN]={0};
uint16_t real_pwm[PWM_PIN]={0};
uint16_t old_pwm[PWM_PIN]={false};
bool H_update=false; 

//--------------------------------------------------------
void PWM(uint8_t pin, uint16_t pwm) {
 FOR_i(0,PWM_PIN) {
   if ((i==pin) || (pin>=PWM_PIN)) {
     signal_pwm[i] = pwm;
     }
   }
 H_update=true; 
}
//--------------------------------------------------------
void PWM_inc(uint8_t pin, int8_t pwm) {
 FOR_i(0,PWM_PIN) {
   if ((i==pin) || (pin>=PWM_PIN)) {
     if (pwm>0 && signal_pwm[i]<1023) signal_pwm[i]++; else module_parsig(8,"PWM MAX");
     if (pwm<0 && signal_pwm[i]>0) signal_pwm[i]--; module_parsig(8,"PWM MIN");
     }
   }
 H_update=true; 
}
//--------------------------------------------------------
void setup_PWM() {
  FOR_i(0,PWM_PIN) pinMode(pin_pwm[i],OUTPUT);
}
//--------------------------------------------------------
void loop_PWM() {
  EVERY_MS(50) {
    FOR_i(0,PWM_PIN)
    {
     if (real_pwm[i]>signal_pwm[i]) real_pwm[i]--;
     if (real_pwm[i]<signal_pwm[i]) real_pwm[i]++;
     analogWrite(pin_pwm[i],real_pwm[i]);
     }
  }
}
//--------------------------------------------------------
// Парсит команды из Ararm,Сети,ИК
void parsing_PWM(char act_chr[]) {
    memcpy(old_pwm, signal_pwm, PWM_PIN);
    String act=String(act_chr);
    uint16_t pwm;
    String strCommand=getValue(act,' ',0);
    String param;
    strCommand.toUpperCase();
    H_update=false; 
    if ((strCommand=="PWM") || (strCommand=="ШИМ") || (strCommand=="Шим")) {
        param = getValue(act,' ',1);
        uint8_t pin=param.toInt();
        param = getValue(act,' ',2);
        if (param=="MIN") return; else
        if (param=="MAX") return; else
        if (param=="+") PWM_inc(pin,1);else
        if (param=="-") PWM_inc(pin,-1);else 
        if (param=="ON") PWM(pin,1023); else 
        if (param=="OFF") PWM(pin,0); else 
        {
           pwm=constrain(param.toInt(),0,1023);
           PWM(pin,pwm);
        }
    }
    // Для совместимости с приложениями GyverLamp
    if (strCommand=="P_ON") PWM(PWM_PIN,1023);
    if (strCommand=="P_OFF") PWM(PWM_PIN,0);
    if (!strncmp_P(act_chr, PSTR("BRI"), 3)) 
    { 
      param = act.substring(3);
      pwm=constrain(param.toInt()<<2,0,1023);
      FOR_i(0,PWM_PIN) PWM(i,pwm);
    }

    if (H_update && hub.focused()) 
      FOR_i(0,PWM_PIN) hub.sendUpdate("PWM"+String(i), String(signal_pwm[i]));

    FOR_i(0,PWM_PIN) {
       char send_udp[25];
       sprintf_P(send_udp, (const char *)F("PWM %d IS %d"), i, real_pwm[i]);
       module_parsig(255,send_udp);
    }
}
//--------------------------------------------------------

void HUB_module_PWM() {
  FOR_i(0,PWM_PIN) {
     hub.WidgetSize(50);
     hub.Display("ШИМ "+String(i),"PWM "+String(pin_pwm[i])+" пин",GH_YELLOW, 1, 18);
     if (hub.Slider_("PWM"+String(i),&signal_pwm[i],GH_UINT16, F("Яркость при включении"), 0, 1023, 1,GH_YELLOW))  PWM(i, signal_pwm[i]) ;
  }
}
//--------------------------------------------------------
void get_PWM() {
 char send_udp[25];
 FOR_i(0,PWM_PIN) {
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    sprintf_P(send_udp, (const char *)F("PWM %d IS %d"), i, real_pwm[i]);
    Udp.write(send_udp);
    Udp.endPacket();
 }   
}  

#endif
